/***************************************************************************
 *            gui.c
 *
 *  Sun Dec 26 16:04:04 2004
 *  Copyright  2004  Ian McIntosh
 *  ian_mcintosh@linuxadvocate.org
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <gdk/gdkx.h>
#include <glade/glade.h>
#include <pango/pangoxft.h>	/* for listing of fonts */

#include "../include/main.h"
#include "../include/prefs.h"

#define CHECK_REDRAW_INTERVAL_MS		(50)		/* check for redraw every X ms */
#define REDRAW_COUNTDOWN_AFTER_UPDATE	(8)			/* redraw this # of checks (above #) after a change */

#define MAX_FONT_MARKUP_LEN 			(4000)		/* max # of bytes for a single line of font text markup */

/* List column numbers */
#define USEDFONTS_COLUMN_COUNT			(2)
#define USEDFONTS_COLUMN_MARKUP			(0)
#define USEDFONTS_COLUMN_FONT_NAME 		(1)

#define UNUSEDFONTS_COLUMN_COUNT	  	(1)
#define UNUSEDFONTS_COLUMN_FONT_NAME 	(0)

typedef struct {
	GtkWindow* m_pWindow;
	GtkColorButton* m_pForegroundColorButton;
	GtkColorButton* m_pBackgroundColorButton;
	GtkSpinButton* m_pFontSizeSpinButton;
	GtkToggleToolButton* m_pBoldButton;
	GtkToggleToolButton* m_pItalicButton;
	GtkToggleToolButton* m_pUnderlineButton;
	GtkEntry* m_pExampleTextEntry;

	/* All fonts will be in one list or the other */
	GtkTreeView* m_pUsedFontsTreeView;
	GtkListStore* m_pUsedFontListStore;	
	GtkTreeView* m_pUnusedFontsTreeView;
	GtkListStore* m_pUnusedFontListStore;

	GtkWidget* m_pUnusedFontListContainer;	/* a container to allow easy hiding/showing */
	GtkButton* m_pAddButton;
	GtkButton* m_pRemoveButton;
/*	GtkButton* m_pRefreshButton; */

	GtkLabel* m_pFontPreviewLabel;

	gint m_nRedrawCountdown;
} MainWindow;

/***************************************************************
 	Prototypes
****************************************************************/

void on_used_font_list_selection_changed(GtkTreeSelection*, gpointer);
void on_unused_font_list_selection_changed(GtkTreeSelection*, gpointer);
gboolean on_timer_check_redraw(gpointer);

/***************************************************************
 	Global Data
****************************************************************/

MainWindow g_MainWindow = {0};


/***************************************************************
 	Functions
****************************************************************/

gboolean gui_find_all_widgets(GladeXML *pGladeXML)
{
	g_MainWindow.m_pWindow = GTK_WINDOW(glade_xml_get_widget(pGladeXML, "mainwindow"));	g_return_val_if_fail(g_MainWindow.m_pWindow != NULL, FALSE);
	g_MainWindow.m_pForegroundColorButton = GTK_COLOR_BUTTON(glade_xml_get_widget(pGladeXML, "textcolor"));	g_return_val_if_fail(g_MainWindow.m_pForegroundColorButton != NULL, FALSE);
	g_MainWindow.m_pBackgroundColorButton = GTK_COLOR_BUTTON(glade_xml_get_widget(pGladeXML, "backgroundcolor")); g_return_val_if_fail(g_MainWindow.m_pBackgroundColorButton != NULL, FALSE);
	g_MainWindow.m_pFontSizeSpinButton = GTK_SPIN_BUTTON(glade_xml_get_widget(pGladeXML, "fontsizespinbutton")); g_return_val_if_fail(g_MainWindow.m_pFontSizeSpinButton != NULL, FALSE);
	g_MainWindow.m_pBoldButton = GTK_TOGGLE_TOOL_BUTTON(glade_xml_get_widget(pGladeXML, "boldbutton")); g_return_val_if_fail(g_MainWindow.m_pBoldButton != NULL, FALSE);
	g_MainWindow.m_pItalicButton = GTK_TOGGLE_TOOL_BUTTON(glade_xml_get_widget(pGladeXML, "italicbutton")); g_return_val_if_fail(g_MainWindow.m_pItalicButton != NULL, FALSE);
	g_MainWindow.m_pUnderlineButton = GTK_TOGGLE_TOOL_BUTTON(glade_xml_get_widget(pGladeXML, "underlinebutton")); g_return_val_if_fail(g_MainWindow.m_pUnderlineButton != NULL, FALSE);
	g_MainWindow.m_pExampleTextEntry = GTK_ENTRY(glade_xml_get_widget(pGladeXML, "exampletext")); g_return_val_if_fail(g_MainWindow.m_pExampleTextEntry != NULL, FALSE);
	g_MainWindow.m_pUsedFontsTreeView = GTK_TREE_VIEW(glade_xml_get_widget(pGladeXML, "usedfontlist")); g_return_val_if_fail(g_MainWindow.m_pUsedFontsTreeView != NULL, FALSE);
	g_MainWindow.m_pUnusedFontsTreeView = GTK_TREE_VIEW(glade_xml_get_widget(pGladeXML, "unusedfontlist")); g_return_val_if_fail(g_MainWindow.m_pUnusedFontsTreeView != NULL, FALSE);
	g_MainWindow.m_pAddButton = GTK_BUTTON(glade_xml_get_widget(pGladeXML, "addbutton")); g_return_val_if_fail(g_MainWindow.m_pAddButton != NULL, FALSE);
	g_MainWindow.m_pRemoveButton = GTK_BUTTON(glade_xml_get_widget(pGladeXML, "removebutton"));	g_return_val_if_fail(g_MainWindow.m_pRemoveButton != NULL, FALSE);
	g_MainWindow.m_pFontPreviewLabel = GTK_LABEL(glade_xml_get_widget(pGladeXML, "fontpreviewlabel")); g_return_val_if_fail(g_MainWindow.m_pFontPreviewLabel != NULL, FALSE);
	g_MainWindow.m_pUnusedFontListContainer = GTK_WIDGET(glade_xml_get_widget(pGladeXML, "unusedfontlistcontainer")); g_return_val_if_fail(g_MainWindow.m_pUnusedFontListContainer != NULL, FALSE);	
	return TRUE;
}

/* Get/Set for all widgets */
void gui_get_foreground_color(GdkColor* pColor)	{ gtk_color_button_get_color(g_MainWindow.m_pForegroundColorButton, pColor); }
void gui_set_foreground_color(GdkColor* pColor) { gtk_color_button_set_color(g_MainWindow.m_pForegroundColorButton, pColor); }
void gui_get_background_color(GdkColor* pColor) { gtk_color_button_get_color(g_MainWindow.m_pBackgroundColorButton, pColor); }
void gui_set_background_color(GdkColor* pColor) { gtk_color_button_set_color(g_MainWindow.m_pBackgroundColorButton, pColor); }
gint gui_get_font_size() 						{ return gtk_spin_button_get_value_as_int(g_MainWindow.m_pFontSizeSpinButton); }
void gui_set_font_size(gint nFontSize) 			{ gtk_spin_button_set_value(g_MainWindow.m_pFontSizeSpinButton, (gdouble)nFontSize); }
gboolean gui_get_bold() 						{ return gtk_toggle_tool_button_get_active(g_MainWindow.m_pBoldButton); }
void gui_set_bold(gboolean bSet) 				{ gtk_toggle_tool_button_set_active(g_MainWindow.m_pBoldButton, bSet); }
gboolean gui_get_italics() 						{ return gtk_toggle_tool_button_get_active(g_MainWindow.m_pItalicButton); }
void gui_set_italics(gboolean bSet) 			{ gtk_toggle_tool_button_set_active(g_MainWindow.m_pItalicButton, bSet);	}
gboolean gui_get_underline() 					{ return gtk_toggle_tool_button_get_active(g_MainWindow.m_pUnderlineButton); }
void gui_set_underline(gboolean bSet) 			{ gtk_toggle_tool_button_set_active(g_MainWindow.m_pUnderlineButton, bSet); }
const gchar* gui_get_example_text() 			{ return gtk_entry_get_text(g_MainWindow.m_pExampleTextEntry); }
void gui_set_example_text(gchar* pscText) 		{ gtk_entry_set_text(g_MainWindow.m_pExampleTextEntry, pscText); }

void gui_configure_used_fonts_lists()
{
	GtkCellRenderer *pCellRenderer;
  	GtkTreeViewColumn *pColumn;

	/* create the list store and add to treeview*/
	g_MainWindow.m_pUsedFontListStore = gtk_list_store_new(USEDFONTS_COLUMN_COUNT, G_TYPE_STRING, G_TYPE_STRING);
	gtk_tree_view_set_model(g_MainWindow.m_pUsedFontsTreeView, GTK_TREE_MODEL(g_MainWindow.m_pUsedFontListStore));

	/* add a new column that displays markup */
	pCellRenderer = gtk_cell_renderer_text_new();
  	pColumn = gtk_tree_view_column_new_with_attributes("", pCellRenderer, "markup", USEDFONTS_COLUMN_MARKUP, NULL);
	gtk_tree_view_append_column(g_MainWindow.m_pUsedFontsTreeView, pColumn);

	/* enable multi-select */
	gtk_tree_selection_set_mode(gtk_tree_view_get_selection(g_MainWindow.m_pUsedFontsTreeView), GTK_SELECTION_MULTIPLE);	

	/* enable search and set search column */
	gtk_tree_view_set_search_column(g_MainWindow.m_pUsedFontsTreeView, USEDFONTS_COLUMN_FONT_NAME);
	gtk_tree_view_set_enable_search(g_MainWindow.m_pUsedFontsTreeView, TRUE);

	/* attach handler for selection-changed signal */
	GtkTreeSelection *pTreeSelection = gtk_tree_view_get_selection(g_MainWindow.m_pUsedFontsTreeView);
	g_signal_connect(G_OBJECT(pTreeSelection), "changed", (GtkSignalFunc)on_used_font_list_selection_changed, NULL);
}

void gui_update_used_fonts_markup()
{
  	GtkTreeIter iter;
	gchar achTextMarkup[ MAX_FONT_MARKUP_LEN + 1];
	const gchar* pscExampleText;
	const gchar* pscFontFamilyName;

	/* get example text to use */
	pscExampleText = gui_get_example_text();
	if(pscExampleText == NULL || *pscExampleText == '\0') {
		pscExampleText = DEFAULT_EXAMPLE_TEXT;
	}

	/* Empty? */
	if(gtk_tree_model_get_iter_first(GTK_TREE_MODEL(g_MainWindow.m_pUsedFontListStore), &iter) == FALSE) {
		return;
	}

	gint nFontSize = gui_get_font_size();
	gboolean bBold = gui_get_bold();
	gboolean bItalics = gui_get_italics();
	gboolean bUnderline = gui_get_underline();

	/* Determine color for the text showing font details */
	GdkColor clrFontFamilyForeground;
	
	GdkColor clrBackground;
	gui_get_background_color(&clrBackground);
	if((clrBackground.red + clrBackground.green + clrBackground.blue) > (35000*3)) {
		/* Use a dark color */
		clrFontFamilyForeground.red = clrFontFamilyForeground.green = clrFontFamilyForeground.blue = 15000;
	}
	else {
		/* Use a light color */
		clrFontFamilyForeground.red = clrFontFamilyForeground.green = clrFontFamilyForeground.blue = 60000;
	}

	gchar* pscFontFamilyForegroundHex = make_hex_triplet(&clrFontFamilyForeground);

	GdkColor clrForeground;
	gui_get_foreground_color(&clrForeground);
	gchar* pscForegroundHex = make_hex_triplet(&clrForeground);

	do {
		/* get font name from row */
		gtk_tree_model_get(GTK_TREE_MODEL(g_MainWindow.m_pUsedFontListStore),
                                             &iter,
                                             USEDFONTS_COLUMN_FONT_NAME,
                                             &pscFontFamilyName,
											 -1);

		/* format markup for row */
		g_snprintf(achTextMarkup, MAX_FONT_MARKUP_LEN,
			"<span font_family='%s' foreground='%s' font_desc='%d' weight='%s' style='%s' underline='%s'>%s</span>\n<span foreground='%s' size='small'>%s\n</span>", /* extra \n to create some blank space */
			pscFontFamilyName,
			pscForegroundHex,
			nFontSize,	/* font_desc='12' works as a partial font description */
			(bBold) ? "bold": "normal",
			(bItalics) ? "italic" : "normal",
			(bUnderline) ? "single" : "none",
			pscExampleText,
			pscFontFamilyForegroundHex,
			pscFontFamilyName);

		/* set markup for row */
		gtk_list_store_set(g_MainWindow.m_pUsedFontListStore, &iter, USEDFONTS_COLUMN_MARKUP, achTextMarkup, -1);
	}
	while(gtk_tree_model_iter_next(GTK_TREE_MODEL(g_MainWindow.m_pUsedFontListStore), &iter));

	g_free(pscForegroundHex);
	g_free(pscFontFamilyForegroundHex);
}

void gui_configure_unused_fonts_lists()
{
	GtkCellRenderer *pCellRenderer;
  	GtkTreeViewColumn *pColumn;

	/* unused fonts list store */
	g_MainWindow.m_pUnusedFontListStore = gtk_list_store_new(1, G_TYPE_STRING);

	/* add model to tree view */
	gtk_tree_view_set_model(g_MainWindow.m_pUnusedFontsTreeView, GTK_TREE_MODEL(g_MainWindow.m_pUnusedFontListStore));
	
	/* add column with a text renderer */
	pCellRenderer = gtk_cell_renderer_text_new();
  	pColumn = gtk_tree_view_column_new_with_attributes("Available Fonts", pCellRenderer, "text", 0, NULL);	
	gtk_tree_view_append_column(g_MainWindow.m_pUnusedFontsTreeView, pColumn);

	/* set search column */
	gtk_tree_view_set_search_column(g_MainWindow.m_pUnusedFontsTreeView, UNUSEDFONTS_COLUMN_FONT_NAME);
	gtk_tree_view_set_enable_search(g_MainWindow.m_pUnusedFontsTreeView, TRUE);

	/* set multi-select */
	gtk_tree_selection_set_mode(gtk_tree_view_get_selection(g_MainWindow.m_pUnusedFontsTreeView), GTK_SELECTION_MULTIPLE);	

	/* attach handler for selection-changed signal */
	GtkTreeSelection *pTreeSelection = gtk_tree_view_get_selection(g_MainWindow.m_pUnusedFontsTreeView);
	g_signal_connect(G_OBJECT(pTreeSelection), "changed", (GtkSignalFunc)on_unused_font_list_selection_changed, NULL);
}

void gui_populate_unused_font_list()
{
  	GtkTreeIter iter;
	gint nScreen=0;
	PangoFontFamily **ppFamilies = NULL;
	gint nNumFamilies=0;
	GtkWidget* pTopLevelWindow;
	PangoContext* pPangoContext;
	const gchar* pscFontFamilyName;
	gint i;

	/* get list of all font families */
	pTopLevelWindow = gtk_widget_get_toplevel(GTK_WIDGET(g_MainWindow.m_pUsedFontsTreeView));
	g_return_if_fail(pTopLevelWindow != NULL);

	gtk_widget_realize(GTK_WIDGET(pTopLevelWindow));	/* this forces a ->window member to be created */
	g_return_if_fail(pTopLevelWindow->window != NULL);

	pPangoContext = pango_xft_get_context(GDK_WINDOW_XDISPLAY(pTopLevelWindow->window), nScreen);
	g_return_if_fail(pPangoContext != NULL);
	pango_context_list_families(pPangoContext, &ppFamilies, &nNumFamilies);

	for(i=0 ; i < nNumFamilies ; i++) {
		pscFontFamilyName = pango_font_family_get_name(ppFamilies[i]);

		/* append this item */
		gtk_list_store_append(g_MainWindow.m_pUnusedFontListStore, &iter);
		gtk_list_store_set(g_MainWindow.m_pUnusedFontListStore, &iter, UNUSEDFONTS_COLUMN_FONT_NAME, pscFontFamilyName, -1);
	}
	g_free(ppFamilies);	ppFamilies = NULL;
}

void gui_remove_selected_used_fonts()
{
	GtkTreeSelection* pTreeSelection = gtk_tree_view_get_selection(g_MainWindow.m_pUsedFontsTreeView);
	GtkTreeIter iter;
	GtkTreeIter iter2;
	gint nNumberRemoved = 0;

	GtkTreePath* pLastPath = NULL;

	/* attempt to get first */
	if(gtk_tree_model_get_iter_first(GTK_TREE_MODEL(g_MainWindow.m_pUsedFontListStore), &iter)) {
		while(TRUE) {
			/* is it selected? */
			if(gtk_tree_selection_iter_is_selected(pTreeSelection, &iter)) {
				/* yes */

				nNumberRemoved++;

				/* get font name from the user list */
				const gchar* pscFontFamilyName;
				gtk_tree_model_get(GTK_TREE_MODEL(g_MainWindow.m_pUsedFontListStore), &iter, USEDFONTS_COLUMN_FONT_NAME, &pscFontFamilyName, -1);

				/* add it to the unused list */
				gtk_list_store_append(g_MainWindow.m_pUnusedFontListStore, &iter2);
				gtk_list_store_set(g_MainWindow.m_pUnusedFontListStore, &iter2, UNUSEDFONTS_COLUMN_FONT_NAME, pscFontFamilyName, -1);

				/* remove it from the used list (FALSE means it was the last row) */
				if(FALSE == gtk_list_store_remove(GTK_LIST_STORE(g_MainWindow.m_pUsedFontListStore), &iter)) {
					break;
				}

				// save path
				if(pLastPath != NULL) {
					gtk_tree_path_free(pLastPath);
				}
				pLastPath = gtk_tree_model_get_path(GTK_TREE_MODEL(g_MainWindow.m_pUsedFontListStore), &iter);
			}
			else {
				/* not selected -- move to next item and loop (FALSE means it was the last row) */
				if(FALSE == gtk_tree_model_iter_next(GTK_TREE_MODEL(g_MainWindow.m_pUsedFontListStore), &iter)) {
					break;					
				}
			}
		}
	}

	/* Scroll to make new cell visible */
	if(nNumberRemoved > 0) {
		GtkTreePath* pPath = gtk_tree_model_get_path(GTK_TREE_MODEL(g_MainWindow.m_pUnusedFontListStore), &iter2);
		gtk_tree_view_scroll_to_cell(g_MainWindow.m_pUnusedFontsTreeView, pPath, NULL, FALSE, 0.0, 0.0);
		gtk_tree_path_free(pPath );

		/* make sure the row after the rows removed is left selected */
		if(pLastPath != NULL) {
			gtk_tree_view_set_cursor(GTK_TREE_VIEW(g_MainWindow.m_pUsedFontsTreeView), pLastPath, NULL, FALSE);
			gtk_tree_path_free(pLastPath);
		}
	}
}

/* activate selected fonts */
void gui_add_selected_unused_fonts()
{
	GtkTreeSelection* pTreeSelection;
	GtkTreeIter iter;
	GtkTreeIter iter2;
	gint nNumberAdded = 0;

	GtkTreePath* pLastPath  = NULL;

	pTreeSelection = gtk_tree_view_get_selection(g_MainWindow.m_pUnusedFontsTreeView);
	
	/* attempt to get first */
	if(gtk_tree_model_get_iter_first(GTK_TREE_MODEL(g_MainWindow.m_pUnusedFontListStore), &iter)) {
		while(TRUE) {
			/* is it selected? */
			if(gtk_tree_selection_iter_is_selected(pTreeSelection, &iter)) {
				/* yes */
				
				nNumberAdded++;

				/* get font name from the user list */
				const gchar* pscFontFamilyName;
				gtk_tree_model_get(GTK_TREE_MODEL(g_MainWindow.m_pUnusedFontListStore),
													 &iter,
													 UNUSEDFONTS_COLUMN_FONT_NAME,
													 &pscFontFamilyName,
													 -1);
			
				/* add it to the unused list */
				gtk_list_store_append(g_MainWindow.m_pUsedFontListStore, &iter2);
				gtk_list_store_set(g_MainWindow.m_pUsedFontListStore, &iter2, 
						USEDFONTS_COLUMN_FONT_NAME, pscFontFamilyName,
						-1);

				/* remove it from the used list (FALSE means it was the last row) */
				if(FALSE == gtk_list_store_remove(GTK_LIST_STORE(g_MainWindow.m_pUnusedFontListStore), &iter)) {
					break;
				}
				
				// save path
				if(pLastPath != NULL) {
					gtk_tree_path_free(pLastPath);
				}
				pLastPath = gtk_tree_model_get_path(GTK_TREE_MODEL(g_MainWindow.m_pUnusedFontListStore), &iter);
			}
			else {
				/* not selected -- move to next item and loop */
				if(FALSE == gtk_tree_model_iter_next(GTK_TREE_MODEL(g_MainWindow.m_pUnusedFontListStore), &iter)) {
					break;					
				}
			}
		}
	}
	gui_update_used_fonts_markup();
	
	/* Scroll to make new cell visible */
	if(nNumberAdded > 0) {
			GtkTreePath* pPath = gtk_tree_model_get_path(GTK_TREE_MODEL(g_MainWindow.m_pUsedFontListStore), &iter2);
			gtk_tree_view_scroll_to_cell(g_MainWindow.m_pUsedFontsTreeView, pPath, NULL, FALSE, 0.0, 0.0);
			gtk_tree_path_free(pPath );

			/* make sure the row after the rows removed is left selected */
			if(pLastPath != NULL) {
				gtk_tree_view_set_cursor(GTK_TREE_VIEW(g_MainWindow.m_pUnusedFontsTreeView), pLastPath, NULL, FALSE);
				gtk_tree_path_free(pLastPath);
			}
	}
}

/* get color from widget and set window */
void gui_set_used_fonts_background_color()
{
	GdkColor clrBackground;
	gui_get_background_color(&clrBackground);
	gtk_widget_modify_base(GTK_WIDGET(g_MainWindow.m_pUsedFontsTreeView), GTK_STATE_NORMAL, &clrBackground);
}

gboolean gui_init()
{
	glade_gnome_init();

	/* Load and connect glade file */
	GladeXML *pGladeXML = glade_xml_new(PACKAGE_DATA_DIR"/gfontsampler/glade/gfontsampler.glade", NULL, NULL);
	if(pGladeXML == NULL) {
		/* try source directory if user hasn't done a 'make install' (good for development, too!) */
		pGladeXML = glade_xml_new(PACKAGE_SOURCE_DIR"/gfontsampler.glade", NULL, NULL);

		/* still not found? */
		if(pGladeXML == NULL) {
			return FALSE;
		}
		g_print("using glade file '%s'\n", PACKAGE_SOURCE_DIR"/gfontsampler.glade");
	}
	g_return_val_if_fail(pGladeXML != NULL, FALSE);

	glade_xml_signal_autoconnect(pGladeXML);
	gui_find_all_widgets(pGladeXML);

	/* set version number -- NO, we need an about box :/ */
/*	gtk_window_set_title(g_MainWindow.m_pWindow, "Font Sampler - "VERSION); */

	/* set the lists to behave how we want */
	gui_configure_used_fonts_lists();
	gui_configure_unused_fonts_lists();

	/* sort unused font list (and keep it sorted from now on) */
	gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(g_MainWindow.m_pUnusedFontListStore), UNUSEDFONTS_COLUMN_FONT_NAME, GTK_SORT_ASCENDING);

	g_timeout_add(CHECK_REDRAW_INTERVAL_MS,
                  (GSourceFunc)on_timer_check_redraw,
                  (gpointer)NULL);

	// leave focus on unused font list
	gtk_widget_grab_focus(GTK_WIDGET(g_MainWindow.m_pUnusedFontsTreeView));
	return TRUE;
}

void gui_show()
{
	gtk_widget_show(GTK_WIDGET(g_MainWindow.m_pWindow));
}

/*
**  Callbacks
**
*/

void on_used_font_list_selection_changed(GtkTreeSelection *treeselection, gpointer user_data)
{
	/* set "Remove" button sensitive if >0 items selected */
	gtk_widget_set_sensitive(GTK_WIDGET(g_MainWindow.m_pRemoveButton), gtk_tree_selection_count_selected_rows(treeselection) > 0);	
}

/* callback for "Select Fonts" button */
void on_view_unused_fonts_toggled(GtkWidget* pWidget, gpointer punused)
{
	gboolean bOn = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(pWidget));

	if(bOn) {
		gtk_widget_show(g_MainWindow.m_pUnusedFontListContainer);
		gtk_widget_grab_focus(GTK_WIDGET(g_MainWindow.m_pUnusedFontsTreeView));
	}
	else {
		gtk_widget_hide(g_MainWindow.m_pUnusedFontListContainer);
	}
}

/* called by timer */
gboolean on_timer_check_redraw(gpointer pData)
{
	if(g_MainWindow.m_nRedrawCountdown > 0) {
		g_MainWindow.m_nRedrawCountdown--;
		if(g_MainWindow.m_nRedrawCountdown == 0) {
			gui_update_used_fonts_markup();
		}
	}
	return TRUE;
}

void on_background_color_changed(GtkWidget* pWidget, gpointer punused)
{
	gui_set_used_fonts_background_color();
	gui_update_used_fonts_markup();
}

/* Adding fonts */
void on_add_button_clicked(GtkWidget* pWidget, gpointer punused)
{
	gui_add_selected_unused_fonts();
}
void on_unusedfontlist_row_activated(GtkWidget* pWidget, gpointer punused)
{
	gui_add_selected_unused_fonts();
}

/* Removing fonts */
void on_usedfontlist_row_activated(GtkWidget* pWidget, gpointer punused)
{
	gui_remove_selected_used_fonts();
}

void on_remove_button_clicked(GtkWidget* pWidget, gpointer punused)
{
	gui_remove_selected_used_fonts();
}

/* this callback sets the font preview */
void set_font_preview_foreach(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer data)
{
	/* get font name from row */
	const gchar* pscFontFamilyName;
	gtk_tree_model_get(GTK_TREE_MODEL(model), iter, UNUSEDFONTS_COLUMN_FONT_NAME, &pscFontFamilyName, -1);

	/* format and set markup */
	gchar achTextMarkup[ MAX_FONT_MARKUP_LEN + 1];
	g_snprintf(achTextMarkup, MAX_FONT_MARKUP_LEN, "<span font_family='%s' font_desc='11'>%s</span>", pscFontFamilyName, pscFontFamilyName);
	gtk_label_set_markup(g_MainWindow.m_pFontPreviewLabel, achTextMarkup);
}

void on_unused_font_list_selection_changed(GtkTreeSelection *treeselection, gpointer user_data)
{
	/* set "Add" button sensitive if >0 items selected */
	gint nSelectionCount = gtk_tree_selection_count_selected_rows(treeselection);
	gtk_widget_set_sensitive(GTK_WIDGET(g_MainWindow.m_pAddButton), (nSelectionCount > 0));	

	/* set font preview label (based on # of fonts selected) */
	if(nSelectionCount == 0) {
		gtk_label_set_markup(g_MainWindow.m_pFontPreviewLabel, "");
	}
	else if(nSelectionCount == 1) {
		/* using a foreach callback is an easy way to set the font preview :) */
		gtk_tree_selection_selected_foreach(treeselection, set_font_preview_foreach, NULL);
	}
	else {
		gtk_label_set_markup(g_MainWindow.m_pFontPreviewLabel, "<i>(multiple)</i>");
	}
}

void on_attributes_changed(GtkWidget* pWidget, gpointer pUserData)
{
	/* a setting has changed, update immediately */
	gui_update_used_fonts_markup();
}

/* a special delayed version of above callback */
void on_attributes_changed_delayed(GtkWidget* pWidget, gpointer pUserData)
{
	/* a setting has changed, flag that a redraw is necessary */
	if(!prefs_have_been_read()) return;
	g_MainWindow.m_nRedrawCountdown = REDRAW_COUNTDOWN_AFTER_UPDATE;
}

void on_printbutton_clicked(GtkWidget* pWidget, gpointer pUserData)
{
	/* print the listview */
}

gboolean on_font_size_key_press(GtkWidget *widget, GdkEventKey *event, gpointer user_data)
{
/*
	Haven't gotten this to work!  The official value of the spinbox doesn't change, so
	when we update it gets the old value!

	if(!prefs_have_been_read()) return FALSE;
	g_MainWindow.m_nRedrawCountdown = REDRAW_COUNTDOWN_AFTER_UPDATE;
*/
	return FALSE;
}

/* window X button and alt-f4 handler */
gboolean on_application_delete_event(GtkWidget *pWidget, GdkEvent *event, gpointer user_data)
{
	prefs_write();
	gtk_exit(0);
	return TRUE;	/* satisfy compiler */
}

#if 0
/*
	GdkColor clrFontName;
	if((clrBackground.red + clrBackground.green + clrBackground.blue) > (35000*3)) {
		clrFontName.red = clrFontName.green = clrFontName.blue = 15000;
	}
	else {
		clrFontName.red = clrFontName.green = clrFontName.blue = 60000;
	}
*/
#endif
