/***************************************************************************
 *            prefs.c
 *
 *  Sun Dec 26 16:04:04 2004
 *  Copyright  2004  Ian McIntosh
 *  ian_mcintosh@linuxadvocate.org
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <gconf/gconf-client.h>

#include "../include/main.h"
#include "../include/gui.h"

#define GCONF_ROOT "/apps/gfontsampler"

static GConfClient* g_pGConfClient;
static gboolean g_bPrefsRead = FALSE;	/* don't allow any pref SETing until they have been read once */

gboolean prefs_init(int argc, char *argv[])
{
	gconf_init(argc, argv, NULL);

	g_pGConfClient = gconf_client_get_default();
	g_return_val_if_fail(g_pGConfClient != NULL, FALSE);

	return TRUE;
}

gboolean prefs_have_been_read()
{
	return 	g_bPrefsRead;	
}

void prefs_read()
{
	gchar* str;

	/* foreground color */
	GdkColor clrForeground = DEFAULT_FOREGROUND_COLOR;
	if((str = gconf_client_get_string(g_pGConfClient, GCONF_ROOT"/foreground_color", NULL)) != NULL) {
		gdk_color_parse(str, &clrForeground);
	}
	gui_set_foreground_color(&clrForeground);

	/* background color */
	GdkColor clrBackground = DEFAULT_BACKGROUND_COLOR;
	if((str = gconf_client_get_string(g_pGConfClient, GCONF_ROOT"/background_color", NULL)) != NULL) {
		gdk_color_parse(str, &clrBackground);
	}
	gui_set_background_color(&clrBackground);
	gui_set_used_fonts_background_color();
	
	/* ruler length */
	gint nFontSize;
	nFontSize=gconf_client_get_int(g_pGConfClient, GCONF_ROOT"/font_size", NULL);
	if(nFontSize == 0) {
		nFontSize = DEFAULT_FONT_SIZE;
	}
	gui_set_font_size(nFontSize);

	/* bold, italics, underline */
	gui_set_bold(gconf_client_get_bool(g_pGConfClient, GCONF_ROOT"/bold", NULL));
	gui_set_italics(gconf_client_get_bool(g_pGConfClient, GCONF_ROOT"/italics", NULL));
	gui_set_underline(gconf_client_get_bool(g_pGConfClient, GCONF_ROOT"/underline", NULL));

	/* example text */
	gchar* pscExampleText = gconf_client_get_string(g_pGConfClient, GCONF_ROOT"/example_text", NULL);
	if(pscExampleText == NULL) {
		pscExampleText = "";
	}
	gui_set_example_text(pscExampleText);
	
	g_bPrefsRead = TRUE;
}

void prefs_write()
{
	if(!prefs_have_been_read()) return;

	/* foreground color */
	GdkColor clrForeground = {0};
	gui_get_foreground_color(&clrForeground);
	gconf_client_set_string(g_pGConfClient, GCONF_ROOT"/foreground_color", make_hex_triplet(&clrForeground), NULL);

	/* background color */
	GdkColor clrBackground = {0};
	gui_get_background_color(&clrBackground);
	gconf_client_set_string(g_pGConfClient, GCONF_ROOT"/background_color", make_hex_triplet(&clrBackground), NULL);	
	
	/* font size */
	gconf_client_set_int(g_pGConfClient, GCONF_ROOT"/font_size", gui_get_font_size(), NULL);			

	/* bold, italics, underline */
	gconf_client_set_bool(g_pGConfClient, GCONF_ROOT"/bold", gui_get_bold(), NULL);
	gconf_client_set_bool(g_pGConfClient, GCONF_ROOT"/italics", gui_get_italics(), NULL);
	gconf_client_set_bool(g_pGConfClient, GCONF_ROOT"/underline", gui_get_underline(), NULL);

	/* example text */
	gconf_client_set_string(g_pGConfClient, GCONF_ROOT"/example_text", gui_get_example_text(), NULL);	
}
